import java.awt.*;
import java.awt.event.*;                 // For event classes
import javax.swing.*;                    // For Swing components
import javax.swing.table.*;              // For the table classes
import java.sql.*;                       // For JDBC classes

public class InteractiveSQL extends JFrame implements ActionListener
{
  public static void main(String[] args)
  { // Create the application object
    // Set default values for the command line args
    String user     = "guest";
    String password = "guest";
    String url      = "jdbc:odbc:technical_library";
    String driver   = "sun.jdbc.odbc.JdbcOdbcDriver";

    // Up to 4 arguments in the sequence database url,driver url, user ID, password
    switch(args.length)
    {
      case 4:                                 // Start here for four arguments
        password = args[3];
        // Fall through to the next case
      case 3:                                 // Start here for three arguments
        user = args[2];
        // Fall through to the next case
      case 2:                                 // Start here for two arguments
        driver = args[1];
        // Fall through to the next case
      case 1:                                 // Start here for one argument
        url = args[0];
    }

    InteractiveSQL theApp = new InteractiveSQL(driver, url, user, password);
  }

  public InteractiveSQL(String driver, String url, String user , String password)
  {
    super("InteractiveSQL");                           // Call base constructor
    setBounds(0, 0, 400, 300);                         // Set window bounds
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);        // Close window operation
    addWindowListener(new WindowHandler());            // Listener for window close

    // Add the input for SQL statements at the top

    commandPanel.add(label1);
    commandPanel.add(columnsToSelect);
    columnsToSelect.setToolTipText("Enter columns to select");
    
    commandPanel.add(label2);
    commandPanel.add(tablesToSelectFrom);
    tablesToSelectFrom.setToolTipText("Enter tables to select from");

    commandPanel.add(label3);
    commandPanel.add(selectionCriteria);
    selectionCriteria.setToolTipText("Enter row selection criteria (optional)");

    commandPanel.add(label4);
    commandPanel.add(ordering);
    ordering.setToolTipText("Enter ordering criteria (optional)");
    
    commandPanel.add(goButton);
    goButton.setToolTipText("Click to perform query");
    goButton.addActionListener(this);

    getContentPane().add(commandPanel, BorderLayout.NORTH);

    // Add the status reporting area at the bottom
    status.setLineWrap(true);
    status.setWrapStyleWord(true);
    getContentPane().add(status, BorderLayout.SOUTH);

    // Create the menubar from the menu items
    JMenu fileMenu = new JMenu("File");                // Create File menu
    fileMenu.setMnemonic('F');                         // Create shortcut
    clearQueryItem.addActionListener(this);
    exitItem.addActionListener(this);
    fileMenu.add(clearQueryItem);                      // Add clear query item
    fileMenu.add(exitItem);                            // Add exit item
    menuBar.add(fileMenu);                             // Add menu to the menubar
    setJMenuBar(menuBar);                              // Add menubar to the window

    // Establish a database connection and set up the table
    try
    {
      Class.forName(driver);                           // Load the driver
      connection = DriverManager.getConnection(url, user, password);
      statement = connection.createStatement();

      model = new ResultsModel();                  // Create a table model
      JTable table = new JTable(model);            // Create a table from the model
      table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);   // Use scrollbars
  
      resultsPane = new JScrollPane(table);        // Create scrollpane for table
      getContentPane().add(resultsPane, BorderLayout.CENTER);

    }
    catch(ClassNotFoundException cnfe)
    {
      System.err.println(cnfe);                    // Driver not found
    }
    catch(SQLException sqle)
    {
      System.err.println(sqle);                    // error connection to database
    }
    pack();
    setVisible(true);
  }
  
  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if(source == goButton)
      executeSQL();
    else if(source == clearQueryItem)           // Clear query menu item
    {
      columnsToSelect.setText("");              // Clear SQL entries
      tablesToSelectFrom.setText("");
      selectionCriteria.setText("");
      ordering.setText("");
    }
    else if(source == exitItem)                 // Exit menu item
    {
      dispose();                                 // Release the window resources
      System.exit(0);                            // End the application
    }    
  }

  public void executeSQL()
  {
    String query = createQuery();      // Get the SQL statement
    if(query == null)                      // If there's nothing we are done
      return;
    try
    {
      model.setResultSet(statement.executeQuery(query));
      status.setText("Resultset has " + model.getRowCount() + " rows.");
    }
    catch (SQLException sqle)
    {
      status.setText(sqle.getMessage());        // Display error message
    }
  }

  public String createQuery()
  {
    String columns = columnsToSelect.getText();
    String tables = tablesToSelectFrom.getText();

    if ((columns == null) || (tables == null) || (columns.length() == 0) || (tables.length() == 00))
    {
      JOptionPane.showMessageDialog(this, "You must enter values in the first two fields",
                                    "SQL Error", JOptionPane.ERROR_MESSAGE);
      return null;
    }

    StringBuffer query = new StringBuffer("SELECT ");
    query.append(columns);
    query.append(" FROM ");
    query.append(tables);

    String criteria = selectionCriteria.getText();

    if ((criteria != null) && (criteria.length() != 0))
    {
      query.append(" WHERE ");
      query.append(criteria);
    }

    String order = ordering.getText();

    if ((order != null) && (order.length() != 0))
    {
      query.append(" ORDER BY ");
      query.append(order);
    }
    return query.toString();
  }

  class WindowHandler extends WindowAdapter
  {
    // Handler for window closing event
    public void windowClosing(WindowEvent e)
    {
      dispose();                                 // Release the window resources
      System.exit(0);                            // End the application
    }
  }

  JPanel commandPanel = new JPanel();
  JLabel label1 = new JLabel("SELECT");
  JTextField columnsToSelect = new JTextField(15);
  JLabel label2 = new JLabel("FROM");
  JTextField tablesToSelectFrom = new JTextField(15);
  JLabel label3 = new JLabel("WHERE");
  JTextField selectionCriteria = new JTextField(15);
  JLabel label4 = new JLabel("ORDER BY");
  JTextField ordering = new JTextField(15);

  JButton goButton = new JButton("Perform Query");

  JTextArea status = new JTextArea(3,1);       // Output area for status and errors
  JScrollPane resultsPane;

  JMenuBar menuBar = new JMenuBar();                        // The menu bar
  JMenuItem clearQueryItem = new JMenuItem("Clear query");  // Clear SQL item
  JMenuItem exitItem = new JMenuItem("Exit");               // Exit item

  Connection connection;                       // Connection to the database
  Statement statement;                         // Statement object for queries
  ResultsModel model;                          // Table model for resultset
}